function [ampEst] = BSLIM(data, freqPts, phi, NVec, iterMax, alg, q)

% [ampEst] = BSLIM(data, freqPts, phi, NVec, iterMax, alg, q)
%
% Estimate the amplitude spectrum of a medical ultrasound signal using the 
% BSLIM algorithm. The algorithm is published in:
%   Gudmundson et al.: "Blood Velocity Estimation Using Ultrasound and
%   Spectral Iterative Adaptive Approaches", Signal Processing, 2011.
%   DOI: 	10.1016/j.sigpro.2010.12.014
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% IN:
%   data    : KxN matrix of data, where K is the number of samples along 
%             depth (fast-time samples) and N is the number of emissions 
%             (slow-time samples).
%   freqPts : The number of frequency points in which to compute the 
%             amplitude spectrum.
%   phi     : Phase shift due to depth
%   NVec    : Vector pointing out the time of the available samples.
%   iterMax : Number of iterations for IAA
%   alg     : 1-Average over fast-time data
%             2-Average over cov. matrix (Recommended)
%             3-Average of amplitudes (very slow)
%   q       : Type of norm to use (0 <= q <= 1) (Recommended: 1)
%
% OUT:
%   ampEst  : Amplitude estimate.
%
% Erik Gudmundson, 2009-12-17
%


NVec = NVec(:);
dataTmp = data(:,NVec);
[K,N] = size(dataTmp);

%Generate the demodulating matrix
Dinv = repmat(exp(-i*phi*(0:K-1)).',1,N);

%Get the demodulated data
z = Dinv.*dataTmp;
%We want the data as col. vectors
z = transpose(z);

%Generate the 'Fourier matrix'
psiVec = 2*pi*(0:freqPts-1)'/freqPts;
A = exp(i*NVec*psiVec.');

if (alg == 1)
  zMean = mean(z,2);
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialize using least squares
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (alg == 2 || alg == 3)
  ampEstMat = abs(A'*z/N).^2; %Individual estimates per fast-time
else %Here, alg == 1
  ampEst = abs(A'*zMean/N).^2;
end
eta = 1e-10;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Do the iterations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if alg == 3 
  eta = ones(K,1)*eta;
  for iter = 1:iterMax
    for k = 1:K
      P = diag( abs(ampEstMat(:,k)).^(2-q) );
      ampEstMat(:,k) = P*A'*inv(A*P*A' + eta(k)*eye(N))*z(:,k);
      eta(k) = norm(z(:,k) - A*ampEstMat(:,k))^2/N;
    end
  end
  ampEst = mean(ampEstMat,2);
  
elseif alg == 2
  for iter = 1:iterMax   
    P = diag(mean( (sqrt(conj(ampEstMat).*ampEstMat)).^(2-q), 2));
    ampEstMat = P*A'*inv(A*P*A' + mean(eta)*eye(N))*z;
    
    eta = diag((z - A*ampEstMat)'*(z - A*ampEstMat))/N;
  end
  
  %The final step
  ampEst = mean(ampEstMat,2);

elseif alg == 1
  for iter = 1:iterMax
    P = diag( abs(ampEst).^(2-q) );
    ampEst = P*A'*inv(A*P*A' + eta*eye(N))*zMean;
    eta = norm(zMean - A*ampEst)^2/N;
  end
end



