function [bpgCap, bpgPsc, bpgApe1, bpgApe2] = ...
  pgFiltBankMethods(...
  data, gapSize, nbrOfBlocks, NcTil, NsTil, nbrOfFreqs, phi)

% [bpgCap, bpgPsc, bpgApe1, bpgApe2] = ...
%  pgFiltBankMethods(...
%  data, gapSize, nbrOfBlocks, NcTil, NsTil, nbrOfFreqs, phi)
%
% Estimation of the square of the amplitude spectra of 
% periodically gapped blood velocity signals using 
% PG-Capon and PG-APES, i.e., BPG-Capon and BPG-APES
% Note that we have two version of APES:
% APES v1: gVec is the mean of the gVec's for all depths;
% APES v2: ggH is the mean of the ggH for all depths, where
%          ggH = gVec*gVec'.
%
% IN:
%    data       : the data matrix, x-direction denotes
%                 slow time and the y-direction represents
%                 depth (RF-averaging), including missing data.
%    gapSize    : size of gap between blocks
%    nbrOfBlocks: the number of blocks of samples in the data
%    NcTil      : number of blocks (clusters) to filter over
%    NsTil      : number of samples per cluster to filter over
%    nbrOfFreqs : the number of points in the spectrum
%    phi        : phase shift per depth
%
% OUT:
%    bpgCap     : the BPG Capon estimate (amp. squared)
%    bpgPsc     : the BPG Power Capon estimate
%    bpgApe1    : the BPG APES estimate, v1 (amp. squared)
%    bpgApe2    : the BPG APES estimate, v2 (amp. squared)
%
% By Erik Gudmundson, Dec. 2009
%


[nbrOfDepths,NpxNc] = size(data);
Np = NpxNc/nbrOfBlocks; %Data per block
%Nc is nbrOfBlocks
Ns = Np - gapSize; %Total length of each block

%We want the data as col. vectors
Y = transpose(data);

%Number of snapshots
L = (Ns - NsTil + 1)*(nbrOfBlocks - NcTil + 1);

%Vector of samples per snapshot
mVec = repmat( (0:NcTil-1)*Np, NsTil, 1) + ...
  repmat( (0:NsTil-1)', 1, NcTil);
mVec = mVec(:);

klVec = repmat( (0:(nbrOfBlocks-NcTil))*Np, Ns-NsTil+1, 1) + ...
  repmat( (0:Ns-NsTil)', 1, (nbrOfBlocks-NcTil)+1);
klVec = klVec(:);

X = [];
Xd = zeros(NsTil*NcTil,L);
for d = 1:nbrOfDepths
  for l = 1:L
    Xd(:,l) = Y(mVec+klVec(l)+1,d);
  end
  X = [X Xd];
end

%Cov. matrix and its inverse
RHat = X*X'/(nbrOfDepths*L);
RinvHat = inv(RHat);

%Vector of considered frequencies
omega = 2*pi*( (0:nbrOfFreqs-1)'/nbrOfFreqs - .5 );

bpgCap = zeros(nbrOfFreqs,1);
bpgApe1 = zeros(nbrOfFreqs,1);
bpgApe2 = zeros(nbrOfFreqs,1);
bpgPsc = zeros(nbrOfFreqs,1);
gVecPerDepth = zeros(NsTil*NcTil,nbrOfDepths);
for w = 1:nbrOfFreqs
  %Fourier vector of length M=NsTil*NcTil
  aMVec = exp( i*omega(w)*mVec );
  %e = aMVec;
  
  %Fourier vector of length L*nbrOfDepths
  aLVec = [];
  for k=1:nbrOfDepths
    aLVec = [aLVec; exp( i*omega(w)*klVec + i*(k-1)*phi )];
  end
  
  %Depth-dependent Fourier transform of data
  gVec = 1/(nbrOfDepths*L)*X*conj(aLVec);
  %Capon amp. squared estimate
  bpgCap(w) = abs(aMVec'*RinvHat*gVec/(aMVec'*RinvHat*aMVec)).^2;
  %Capon power spectrum estimate
  bpgPsc(w) = 1/abs(aMVec'*RinvHat*aMVec);

  %APES v1: gVec is the mean of the gVec's for all depths
  QHat = RHat - gVec*gVec'/nbrOfDepths;
  QinvHat = inv(QHat);
  
  bpgApe1(w) = abs(aMVec'*QinvHat*gVec/(aMVec'*QinvHat*aMVec)).^2;
  
  %APES v2: ggH is the mean of the ggH for all depths
  ggH = zeros(NsTil*NcTil);
  for k = 1:nbrOfDepths
    gVecPerDepth(:,k) = X(:,((k-1)*L+1):((k-1)*L+L))*...
      conj( aLVec( ((k-1)*L+1):((k-1)*L+L) ))/L;
    ggH = ggH + gVecPerDepth(:,k)*gVecPerDepth(:,k)';
  end
  Q2Hat = RHat - ggH/nbrOfDepths;
  Q2invHat = inv(Q2Hat);
  bpgApe2(w) = abs(aMVec'*Q2invHat*gVec/(aMVec'*Q2invHat*aMVec)).^2;
  
end


