function [per,bpc,bpcFB,bac,bacFB,ape,apeFB,omega] = spec_est_3(Y,no_filt,no_freq,w_t)

% This function estimates the powerspectrum of an 
% ultrasound signal using the Periodogram, Minimum
% Variance and APES.
%
% Function call: [per,psc,ape] = spec_est(Y,no_freq) 
%
% Input: Y       - the data matrix, x-direction denotes
%                  slow time and the y-direction represents
%                  depth (RF-averaging) 
%        no_filt - number of coefficients in the filter
%        no_freq - the number of points in the spectrum
%
% Output: per    - the periodogram
%         psc    - the Minimum Variance estimate
%         ape    - the APES estimate
%
% FG 2006-08-17
% AJ 2007-05-31


Y=transpose(Y);

M  = size(Y,1);   %Data length
K  = size(Y,2);   %Number of depths
J  = fliplr( eye(no_filt,no_filt) );

R=0;
for no=1:M-no_filt+1
  Ysub = Y(no:no+no_filt-1,:);
  R  = R + Ysub*Ysub';       %Corr matrix
end;
R  = 1/((M-no_filt+1)*K)*R;
R2 = ( R+J*R.'*J )/2;
Ri = inv(R);      %Inverse
R2i = inv(R2);

m  = [0:no_filt-1]';

omega = 2*pi*([0:no_freq-1]'/no_freq-1/2);
%omega = 2*pi*([0:no_freq-1]'/no_freq);

for no = 1:length(omega)
  %e = exp( -i*omega(no)*m );
  e = exp( i*omega(no)*m );
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Estimate signal component
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  sumG = zeros(no_filt,no_filt);
  sumP = sumG;
  for k=1:K
    
      % Forward data
      G{k} = zeros(no_filt,M-no_filt);
      for p=1:M-no_filt+1
          %G{k}(:,p) = Y(p:p+no_filt-1,k).*exp( -i*(w_t*(k-1)-omega(no)*((p-1))));
          G{k}(:,p) = Y(p:p+no_filt-1,k).*exp( -i*( w_t*(k-1) + omega(no)*(p-1) ) );
      end 
      g{k} = mean(G{k},2); 
      sumG = sumG + 1/K*g{k}*g{k}';

      % Backward data
      P{k} = zeros(no_filt,M-no_filt);
      for p=1:M-no_filt+1
          %P{k}(:,p) = conj( Y(M-p+1:-1:M-p-no_filt+2,k) ).*exp(-i*(w_t*(k-1)-omega(no)*((p-1))));
          P{k}(:,p) = conj( Y(M-p+1:-1:M-p-no_filt+2,k) ).*exp(-i*(w_t*(k-1)+omega(no)*((p-1))));
      end 
      pp{k} = mean( P{k},2 ); 
      Yz  = [ g{k} pp{k} ] / sqrt(2);
      sumP = sumP + Yz*Yz'/K;
      
  end
  
  Q = R - sumG;
  Qi=inv(Q);
  h = Qi*e / ( e'*Qi*e );

  Qi2 = inv( R2 - sumP );
  h2  = Qi2*e / ( e'*Qi2*e );

  h3  = R2i *e / ( e'*R2i*e );
  h4  = Ri *e / ( e'*Ri*e );

    
  
  per(no) = abs(e'*R2*e);
  bpc(no) = 1/abs(e'*Ri*e);
  bpcFB(no) = 1/abs(e'*R2i*e);
  ape(no)=0;
  bac(no)=0;
  bacFB(no)=0;
  apeFB(no)=0;
  for k=1:K,
      ape(no) = ape(no) + abs( h' * g{k} ).^2;
      apeFB(no) = apeFB(no) + abs( h2' * g{k} ).^2;
      bac(no) = bac(no) + abs( h4' * g{k} ).^2;
      bacFB(no) = bac(no) + abs( h3' * g{k} ).^2;
  end

  
end

ape   = ape/ K;
apeFB = apeFB/K;
bac   = bac/K;
bacFB = bacFB/K;

